import os
import sys
import json
import time
import signal
from typing import List
from pprint import pprint


# =========================================================== display
def p(s):
    print(s)


def pp(d: dict):
    pprint(d)

def sleep(countdown:int):
    time.sleep(countdown)
    
# =========================================================== IO
def jsonread(file_name: str) -> dict:
    res = {}
    with open(file_name, 'r') as f:
        res = json.loads(f.read())
    return res


def jsonwrite(d: dict, file_name: str):
    json.dump(d, open(file_name, 'w'), ensure_ascii=False, indent=2)


# =========================================================== Decorator
def set_timeout(countdown: int, callback=print):
    def decorator(func):
        def handle(signum, frame):
            raise RuntimeError

        def wrapper(*args, **kwargs):
            try:
                signal.signal(signal.SIGALRM, handle)
                signal.alarm(countdown)  # set countdown
                r = func(*args, **kwargs)
                signal.alarm(0)  # close alarm
                return r
            except RuntimeError as e:
                print(e)
                callback()

        return wrapper

    return decorator



# coding:utf-8
import logging
import os
import inspect
from logging.handlers import RotatingFileHandler
import colorlog  # 控制台日志输入颜色

log_colors_config = {
    'DEBUG': 'cyan',
    'INFO': 'green',
    'WARNING': 'yellow',
    'ERROR': 'red',
    'CRITICAL': 'red',
}


class Logger:
    def __init__(self, logname: str = 'log.local.log'):
        self.logname = logname
        self.logger = logging.getLogger()
        self.logger.setLevel(logging.DEBUG)
        self.formatter = colorlog.ColoredFormatter(
            '%(log_color)s[%(asctime)s] [%(levelname)s]- %(message)s',
            log_colors=log_colors_config)

        if self.logger.hasHandlers(): # To above duplicated lines
            return

        # 创建一个 FileHandler，写到本地
        fh = logging.handlers.TimedRotatingFileHandler(self.logname,
                                                       when='MIDNIGHT',
                                                       interval=1,
                                                       encoding='utf-8')
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(self.formatter)
        self.logger.addHandler(fh)

        # 创建一个StreamHandler,写到控制台
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        ch.setFormatter(self.formatter)
        self.logger.addHandler(ch)

    @staticmethod
    def __get_call_info():
        stack = inspect.stack()
        cur = stack[3]
        fn, ln, func = cur[1:4]
        return fn, func, ln

    def console(self, level: str, message: str, *args):
        LV = {
            'debug': self.logger.debug,
            'info': self.logger.info,
            'warn': self.logger.warning,
            'error': self.logger.error,
            'critical': self.logger.critical
        }
        f = LV.get(level, self.debug)
        message = "[{}.{}:{}] {}".format(*self.__get_call_info(), message)
        f(message, *args)

    def debug(self, message):
        self.console('debug', message)

    def info(self, message):
        self.console('info', message)

    def warning(self, message):
        self.console('warning', message)

    def error(self, message):
        self.console('error', message)

    def critical(self, message):
        self.console('critical', message)


def test():
    frames = inspect.stack()
    print(frames)
    Logger().info("Line 6666")


if __name__ == "__main__":
    log = Logger()
    log.info("测试1")
    log.debug("测试2")
    log.warning("warning")
    log.error("error")
    log.critical("critical")
    test()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="xiu",
    version="0.0.1",
    author="Gaogle",
    author_email="byteleap@gmail.com",
    description="A personal package for faster Python programming",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/GaoangLiu/slipper",
    package_dir={'': 'src'},
    py_modules=['xiu', 'logger'],
    # packages=setuptools.find_packages(),
    install_requires=[
        'colorlog>=4.6.1'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
